#include <limits.h>

#include <google/gflags.h>

#include "log.h"
#include "geom.h"
#include "util.h"

DEFINE_double(ignorable_angle, M_PI * 0.5, "");

HalfLine::HalfLine(float dx, float dy, float sx, float sy) {
    this->sx = sx;
    this->sy = sy;
    this->dir = atan2(dy - sy, dx - sx);

    float x = dx - sx;
    float y = dy - sy;
    if (x == 0) {
        a = tan(M_PI/2);
    }
    else {
        a = y / x;
    }
    b = sy - sx * a;
    LOG2("multi line %f*x+%f\n", a, b);
}

HalfLine::HalfLine(float dir, float sx, float sy) {
    this->sx = sx;
    this->sy = sy;
    this->dir = dir;

    a = tan(dir);
    b = sy - sx * a;
    LOG2("multi line %f*x+%f\n", a, b);
}


float HalfLine::distance(float x, float y) const {
    float dir = atan2(y - sy, x - sx);
    float dd = normalizeDir(dir - this->dir);
    if (dd > FLAGS_ignorable_angle || dd < -FLAGS_ignorable_angle) {
        return INT_MAX;
    }

    if (a == 0) {
        return fabs(y - b);
    }
    else {
        float c = -1.0 / a;
        float d = y - c * x;
        float tx = (d - b) / (a - c);
        float ty = c * tx + d;
        return ::distance(x, y, tx, ty);
    }
}
