#include <string>
#include <vector>
#include <utility>

using namespace std;

class Stream;
class World;

struct Obj {
    Obj(char type0, float x0, float y0, float r0);
    Obj(float x0, float y0, float dir0, float speed0);

    char type;
    float x;
    float y;
    float r;
    float dir;
    float speed;
    float distance;
};

class Rover {
public:
    explicit Rover(Stream* stream);

    void init(float dx,
              float dy,
              int time_limit,
              float min_sensor,
              float max_sensor,
              float max_speed,
              float max_turn,
              float max_hard_turn);

    void reset();

    void setState(int timestamp, char accState, char turnState,
                  float x, float y, float dir, float speed);

    void reportCrash(int timestamp);
    void reportFail(char why, int timestamp);
    void reportSuccess(int timestamp);
    void reportDone(int timestamp);

    void clearObjects();

    void addObject(Obj* obj);

    void sortObject();

    void calcBestGoal(float* gx, float* gy);

    void gotoXY(float x, float y);

    void updateTarget();

    void proc();

    void turnR();
    void turnL();

    void guessAccel(int t, float s);
    bool guessRotAccel(int t, float d, float ts);

    void send(const string& ctrl, bool suppressDup = false);

    int currentTime();

    void guessPos(float elapsed = 0.0);

    void replayCmds();

    bool checkNearGoalMissing();

    bool danger();
    bool nearGoal();

    void adjustNearOverlap();

    bool outOfBounds(float x, float y) const;

    bool avoidEmergency(char* accAtt, char* turnAtt);

    bool avoidEmergency2(char origTurnState,
                         char* accAtt,
                         char* desiredTurnState);

    int simulateSafety(float sim_x, float sim_y,
                       float sim_speed, float sim_dir, float sim_rot,
                       float accAtt, float turnAtt,
                       const vector<Obj*>& nearObjs);

    int simulateSafety2(float sim_x, float sim_y,
                        float sim_speed, float sim_dir, float sim_rot,
                        char origTurnState,
                        char accAtt, char turnAtt,
                        const vector<Obj*>& nearObjs);

    void simulateDir(float elapsed, char ts, float* rot, float* dir);

    Stream* stream;

    bool inited;
    bool firstTel;

    // initial states;
    float dx;
    float dy;

    int state_time;
    long long state_real_time;

    int time_limit;
    float min_sensor;
    float max_sensor;
    float max_speed;
    float max_turn;
    float max_hard_turn;

    char accState;
    char turnState;
    float x;
    float y;
    float dir;
    float speed;
    float rot;

    vector<Obj*> objs;
    Obj* goal;

    int mode;

    float targetX;
    float targetY;

    char turnAttempt;

    vector<pair<int, string> > emittedCmds;
    vector<pair<int, string> > prevCmds;

    float guessedAccel;
    bool canGuessAccel;
    bool canGuessRotAccel;
    float guessedRotAcc;
    float prevDirChange;
    bool rotAccelFirstGuess;

    int prev_cmd_time;
    string prev_cmd;
    float prev_speed;

    bool should_brake;

    bool prev_right;
    bool prev_left;

    World* world;

    float gx, gy;

    float reported_dir;
    char reported_turn_state;

    bool avoiding_emergency;
    bool will_die;
};
