#include <stdio.h>

#ifdef DEBUG

# include <SDL.h>
# include <SDL_gfxPrimitives.h>

SDL_Surface* scr;

#endif

#include "log.h"
#include "rover.h"
#include "world.h"
#include "visualizer.h"

using namespace std;

float map_x;
float map_y;

Visual::Visual(Rover* rover0)
    : rover(rover0) {
#ifdef DEBUG
    SDL_Init(SDL_INIT_VIDEO);

    scr = SDL_SetVideoMode(640, 640, 32, SDL_SWSURFACE);
    CHECK(scr);
#endif

    LOG("Visualizer initialized\n");
}

#ifdef DEBUG
static int convX(float x) {
    return (x + map_x / 2) / map_x * 640;
}

static int convY(float y) {
    return (map_y / 2 - y) / map_y * 640;
}

static void drawCircle(float x, float y, float r, Uint32 col) {
    r = max(1, (int)(r / map_x * 640));
    filledCircleColor(scr, convX(x), convY(y), r, col);
}
#endif

void Visual::draw() {
#ifdef DEBUG
    map_x = rover->dx;
    map_y = rover->dy;

    SDL_FillRect(scr, NULL, SDL_MapRGB(scr->format, 255, 255, 255));

    drawCircle(0, 0, 5, 0x00ff00ff);

    if (rover->will_die) {
        drawCircle(rover->x, rover->y, 10, 0xff0000ff);
    }
    else if (rover->avoiding_emergency) {
        drawCircle(rover->x, rover->y, 10, 0xffff00ff);
    }

    if (rover->prev_cmd.find('b') != string::npos) {
        drawCircle(rover->x, rover->y, 3, 0xff00ffff);
    }
    else {
        drawCircle(rover->x, rover->y, 3, 0xff0000ff);
    }

    lineColor(scr,
              convX(rover->x), convY(rover->y),
              convX(rover->world->gx), convY(rover->world->gy), 0xdd0000ff);
    lineColor(scr,
              convX(rover->world->gx), convY(rover->world->gy),
              convX(rover->world->gx2), convY(rover->world->gy2), 0xdd0000ff);
    lineColor(scr,
              convX(rover->x), convY(rover->y),
              convX(rover->gx), convY(rover->gy), 0xdd00ddff);

    for (int i = 0; i < rover->objs.size(); i++) {
        Obj* o = rover->objs[i];
        if (o->type != 'm') continue;
        drawCircle(o->x, o->y, 1, 0x0000ffff);
    }

    for (int i = 0; i < rover->world->cluster.size(); i++) {
        const vector<WO*>& c = rover->world->cluster[i];
        for (int j = 0; j < c.size(); j++) {
            Uint32 col;
            if (c[j]->type == 'c') {
                col = 0xdd4444ff;
            }
            else {
                col = 0x444444ff;
                //col = SDL_MapRGB(scr->format, 200, 100, 100);
            }
            drawCircle(c[j]->x, c[j]->y, c[j]->r, col);
        }
    }

    SDL_Flip(scr);
#endif
}
